'' =================================================================================================
''
''   File....... jm_kaboom_blinker.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... 
''   Started....
''   Updated.... 30 JAN 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us

  
  TRM_BAUD = 115_200

  
con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30

  SDA  = 29                                                     ' eeprom / i2c
  SCL  = 28

  LED2 = 18
  LED1 = 17
  LED0 = 16
  

con

  #-1, ON, OFF                                                  ' digital control

  #0, LSBFIRST, MSBFIRST    


obj

' main                                                          ' *   master Spin cog
  time    : "jm_time"                                           '     timing and delays
  prng    : "jm_prng"                                           '     pseudo-random number generator
  pst     : "Parallax Serial Terminal"                          ' *   for PST io
' blinker                                                       ' *** led blinking
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

  long  cog0
  long  stack0[16]

  long  cog1
  long  stack1[16]

  long  cog2
  long  stack2[16]
                                                                   
                                                                 
pub main | delay

  setup

  cog0 := cognew(blinker(LED0, 100, 900), @stack0) + 1          ' start blinkers
  cog1 := cognew(blinker(LED1, 125, 125), @stack1) + 1
  cog2 := cognew(blinker(LED2, 333, 667), @stack2) + 1

  time.pause(2000)                                              ' time to open PST
  pst.clear

  repeat
    pst.str(string("Kaboom asks vague questions...", 13))
    pst.str(string("Kaboom abuses the forums signature line...", 13))
    pst.str(string("Don't be like Kaboom.", 13, 13, 13))

    delay := (prng.random >> 1) // 1001 + 1000                  ' wait 1-2 seconds
    time.pause(delay)
                                                           
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  prng.seed(-cnt, -cnt ~> 2, $EA7_BEEF, cnt << 2, cnt)          ' seed randomizer
                                                                 
  pst.start(TRM_BAUD)                                           ' start PST                                                               


pri blinker(pin, mson, msoff) | t                               ' launch with cognew() only!

  mson *= MS_001                                                ' convert to system ticks
  msoff *= MS_001

  dira[pin] := 1                                                ' set pin to output in this cog
  
  t := cnt                                                      ' sync loop timer
  repeat
    outa[pin] := 1                                              ' pin high
    waitcnt(t += mson)                                          ' hold for on milliseconds
    outa[pin] := 0                                              ' pin low
    waitcnt(t += msoff)                                         ' hold for off milliseconds         


dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}